//Copyright by Shuangge Ma
//This is the hearder file of the EndNormal.cpp

//EndNormal.cpp is the calculation of the one and two endpoint Normal type data

//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//@@ THE ZERO PART: Definitions and Include    @@
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//
// Standard Includes
//
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>

#define LIMIT_N_LOOK 10

#define Pi 3.1415926
#define QuantileSim  2000 //the number of simulation we need for the calculation of Normal quantile.

// The following is the functions and definitions of the class called EndNormal:

class EndNormal
{
public:
	EndNormal();		//constructor

// Definitions of functions.
	
	int one_stage_normal(void);
	//Based on input information, get the sample size estimate for one-stage-normal
	//We will store the value in the EndNormal structure.

	void information(void);
	//get inform_time from event_look;

	double *corr_one_normal(void);
	//Based on the EndNormal.inform_time, we get the covariance matrix 
	//of the uni normal endpoint.
	//return value is a matrix.

	double *NullAlterGenerator1();
	//Based on the information in the EndNormal, we generate the Null and Alternative in a matrix
	//The matrix will have the structure:
	//		look1		look2			look3			.......
	//	Null, Alter		Null, Alter		Null, Alter		.......
	//We consider the scale parameter only, since the scale parameter is the same for different alternative
	//mean and sample size;



	// SIZE MATCH:
	// The return of NullAlterGenerator is [2*num_look, num_simul]



	double *NullAlterGenerator2(int, double*, double*);
	//From the previous code, we consider the location parameter here.
	//Input: int: trial sample size;
	//		 double *: meanNullAlter,  (mean of Null, mean of Alter)
	//		 double *: the matrix from *NullAlterGenerator1();
	//Output: Null and Alter with location and scale parameters.

	double *alpha_beta1(double);
	//The alpha and beta spending function,
	//Input: double: overall alpha or beta;
	//Output: alpha[i] or beta[i]

	double *alpha_beta2(double);
	//The same function as previous one.

	double *alpha_beta3(double);

	double *Symmetric_Boundary1(int ,double , double *);
	//since we have information externally, we do not need input here.
	//The calculation of critical boundaries for each endpoint.
	//Input: int trysize2: the trial sample size;
	//       double meanAlter1: the mean of the Alternative.
	//		 double *workMatrix : The NullAlter (with scale parameter only)
	//							  we will input the workMatrix into the NullAlterGenerator2 to get the Null and Alter.
	//Output: an array with both lower bound and upper bound in it.

	void SizeAdjust();
	//int SizeAdjust();
	//After getting the initial sample size estimate, we call this function to adjust sample size 
	//and get the critical boundaries.
	//When we run this function, it calls the Symmetric_Boundary1 (which in turn calls those NullAlterGenerators)
	//No output needed. we will store information in the structure.

	double *symm_prob_count(double *, double *,double *, double *);
	//We count the prob in each region for the symmetric case.
	//which allows hard decisions ONLY.
	//Input: double* boundary1: critical boundary for the first endpoint
	//       double* simul_matrix1: the simulated NullAlter from the NullAlterGenerator1();
	//       double* boundary2: critical boundary for the second endpoint
	//		 double* simul_matrix2: the simulated NullAlter for the second endpoint.

	double *nonsymm_prob_count(double *, double *, double *, double *);
	//The same function as previous one. for the non-symmetric case
	//which allows soft decisions for the second endpoint.

	void Design_Symm_Normal();
	//The design of two endpoint Normal case.
	//Search for the constant c to satisfy the overall alpha requirement
	//Then count the beta for each Alternative to get the worst case
	//See whether this beta satisfy the requirement, if not, adjust sample size
	//which calls the symm_prob_count() function

	void Design_NonSymm_Normal();
	//The design for the Non-Symmetric Normal Endpoints case.
	//which calls the nonsymm_prob_count function.

	void getLookSimul();
	void makeBoundary();

////////////////////////////////////
//Object variables 

	int order;                 // The first or the second endpoint
	int num_look;				 // number of look
	char *type;         // data type: Normal or Survival
	int num_simul;             // total number of simulations. look_simul=num_simul/num_look
	int look_simul;            // number of simulation for each look.
	
	int rule_decision;		// decision: hard--0 or soft--1

	double mean_Null;
	double mean_Alter;
	double var_Null;
	double var_Alter;			 // Mean and variance under Null and Alternative hypothesis
	
	int *event_look;     // The number of events that are expected to happen before each look. 
	double *inform_time;   // information spending.

	double overall_alpha;
	double overall_beta;
	int spending_num;
	//spending is interger taype variable. means which spending
	//function the clinet choose.


	double *alpha;
	double *beta;

	double *upper_bound;
	double *lower_bound;
	double *boundary;
	
	int sample_size;  // minimal sample size needed for this endpoint
	
	double corr_ph;   // The correlation between two endpoints.   

	double Bi_Alpha;  //the same for two endpoints
	double Bi_Beta;

	int *ctrl_ind;

};


	double *Stand_Normal(int);
	//generation of standard Normal
	//Input: length of target Normal
	//Output: one-dim array

	double maxm(double, double);
	double minm(double, double);
	int maxInt(int, int);
	int minInt(int, int);
	//max and min of two Int variables.
	double absd(double);
	void mySort(double *p, int size);

	double normalscore(double, int);
	//The normal score
	//input: quantile, num of simulations 
	double NormalScoreA(double);
		
	double *sqrt_matrix(int, double*);
	//square root of a semi-pisitive-definite matrix
	//input: size of the matrix, pointer to the matrix
	//out put: pointer to the matrix

	//Then we get the input from the interface.

	double *sqrt_matrixNew(int, double *);
	// The new version of the squre root of a matrix: B' * B = A
	